@@ -178,7 +178,9 @@ class Conv(keras.Model):
     BATCH_NORM='batch'
     GROUP_NORM='group'
     DEFAULT_NORM=BATCH_NORM
-
+    DEFAULT_ORDER=['conv','act','norm','do']
+    ACT_LAST_ORDER=['conv','norm','do','act']
+    RES_ORDER=['norm','act','conv','do']
     #
     # PUBLIC
     #
@@ -196,11 +198,13 @@ class Conv(keras.Model):
             dropout=DEFAULT_DROPOUT,
             dropout_config={},
             act_last=False,
+            order=DEFAULT_ORDER,
             is_skip=False,
             name=None,
             named_layers=True,
             **conv_config):
         super(Conv, self).__init__()
+        self.paddings=False
         self.block_name=name or self.name
         self.named_layers=named_layers
         if seperable:
@@ -213,6 +217,17 @@ class Conv(keras.Model):
             strides=1
         if kernel_size in [1,(1,1)]:
             dilation_rate=1
+            if padding.lower() in ['reflect','symmetric']:
+                padding='valid'
+        elif padding and (padding.lower() in ['reflect','symmetric']):
+            self.padding_mode=padding
+            p=int(kernel_size//2)
+            self.paddings=tf.constant([
+                [0,0],
+                [p,p],
+                [p,p],
+                [0,0]])
+            padding='valid'
         self.conv=_conv(
             filters=filters,
             kernel_size=kernel_size,
@@ -221,23 +236,25 @@ class Conv(keras.Model):
             strides=strides,
             name=self._layer_name(cname),
             **conv_config)
+        self.is_skip=is_skip
+        if order=='res':
+            self.order=Conv.RES_ORDER
+        elif act_last or (order=='act_last'):
+            self.order=Conv.ACT_LAST_ORDER
+        elif isinstance(order,list):
+            self.order=order
+        else:
+            self.order=Conv.DEFAULT_ORDER
         self.norm=self._norm(norm,norm_config)
         self.act=self._activation(act,act_config)
         self.do=self._dropout(dropout,dropout_config)
-        self.act_last=act_last
-        self.is_skip=is_skip
-
-
-    def __call__(self,x,training=False):
-        x=self.conv(x)
-        if self.act_last:
-            if self.norm: x=self.norm(x)
-            if training and self.do: x=self.do(x)
-            if self.act: x=self.act(x)
-        else:
-            if self.act: x=self.act(x)
-            if self.norm: x=self.norm(x)
-            if training and self.do: x=self.do(x)         
+    def __call__(self,x):
+        for m in self.order:
+            l=getattr(self,m)
+            if l: 
+                if (self.paddings is not False) and (m=='conv'):
+                    x=tf.pad(x,self.paddings,self.padding_mode)
+                x=l(x)
         return x
 
 
@@ -305,12 +322,16 @@ class Stack(keras.Model):
             padding='same',
             squeeze_excitation=False,
             squeeze_excitation_ratio=16,
+            channel_squeeze_excitation=False,
             residual=True,
             residual_act=True,
+            residual_norm=None,
+            residual_norm_config=None,
             seperable_residual=False,
             seperable=False,
             norm=True,
             norm_config={},
+            output_dropout=True,
             act=True,
             act_config={},
             input_dilation_rate=1,
@@ -344,6 +365,7 @@ class Stack(keras.Model):
         self._set_config(
             act,
             output_stride,
+            output_dropout,
             dilation_rate,
             max_pooling,
             seperable=seperable,
@@ -357,11 +379,17 @@ class Stack(keras.Model):
             self.se=SqueezeExcitation(
                 filters,
                 ratio=squeeze_excitation_ratio)
+        elif channel_squeeze_excitation:
+            self.se=ChannelSqueezeExcitation(filters)
         else:
             self.se=False
         self.residual=self._residual(
             residual,
             residual_act,
+            residual_norm,
+            residual_norm_config,
+            norm,
+            norm_config,
             self.filters_list[-1],
             output_stride)
         self.stack=self._build_stack(
@@ -417,18 +445,24 @@ class Stack(keras.Model):
     def _set_config(self,
             act,
             output_stride,
+            output_dropout,
             dilation_rate,
             max_pooling,
             seperable,
             seperable_residual,
+            norm,
+            norm_config,
             **shared_config):
         self.act=act
+        self.output_dropout=output_dropout
         self.max_pooling_config=self._max_pooling_config(
             output_stride,
             dilation_rate,
             max_pooling)
         self.seperable=seperable
         self.seperable_residual=seperable_residual
+        self.norm=norm
+        self.norm_config=norm_config
         shared_config['dilation_rate']=dilation_rate
         self.shared_config=shared_config
 
@@ -446,12 +480,27 @@ class Stack(keras.Model):
         return layer_name(self.block_name,name,index=index,named=self.named_layers)
 
 
-    def _residual(self,residual,residual_act,filters,output_stride):
+    def _residual(
+            self,
+            residual,
+            residual_act,
+            residual_norm,
+            residual_config,
+            norm,
+            residual_norm_config,
+            filters,
+            output_stride):
         if residual and (residual!=Stack.IDENTITY):
             if not residual_act:
                 act=False
             else:
                 act=self.act
+            if residual_norm is None:
+                residual_norm=norm
+            if residual_norm_config is None:
+                residual_norm_config=norm_config
+            cfig=self.shared_config.copy()
+            cfig['dropout']=False
             residual=Conv(
                 filters=filters,
                 kernel_size=1,
@@ -460,7 +509,9 @@ class Stack(keras.Model):
                 act=act,
                 name=self._layer_name('residual'),
                 named_layers=self.named_layers,
-                **self.shared_config)
+                norm=residual_norm,
+                norm_config=residual_norm_config,
+                **cfig)
         return residual
 
     
@@ -476,9 +527,12 @@ class Stack(keras.Model):
             dilation_rate=cfig.pop('dilation_rate',1)
             if (i==0):
                 dilation_rate=input_dilation_rate or dilation_rate
-            if (i==last_layer_index) and (not self.max_pooling_config):
-                strides=output_stride or strides
-                dilation_rate=output_dilation_rate or dilation_rate
+            if (i==last_layer_index):
+                if not self.output_dropout:
+                    cfig['dropout']=False
+                if (not self.max_pooling_config):
+                    strides=output_stride or strides
+                    dilation_rate=output_dilation_rate or dilation_rate
             _layers.append(Conv(
                 filters=f,
                 kernel_size=k,
@@ -486,6 +540,8 @@ class Stack(keras.Model):
                 dilation_rate=dilation_rate,
                 seperable=self.seperable,
                 act=self.act,
+                norm=self.norm,
+                norm_config=self.norm_config,
                 name=self._layer_name(self.layers_name,index=i),
                 named_layers=self.named_layers,
                 **cfig))
@@ -707,6 +763,21 @@ class SqueezeExcitation(keras.Model):
 
 
 
+class ChannelSqueezeExcitation(keras.Model):
+    """ ChannelSqueezeExcitation
+    """
+
+    def __init__(self,filters,**squeeze_kwargs):
+        super(ChannelSqueezeExcitation, self).__init__()
+        self.squeeze=layers.Conv2D(1,1,activation='sigmoid',**squeeze_kwargs)
+
+
+    def __call__(self,x):
+        y=self.squeeze(x)
+        return layers.multiply([x,y])
+
+
+
 
 class Residual(keras.Model):
     """ Residual
@@ -798,6 +869,18 @@ class ASPP(Group):
 
 
 
+def get_fixed_initializer(value):
+    """
+    Args: 
+        * value(tensor): a tensor to use for initialization
+    """
+    def _fixed_initializer(shape,dtype=None):
+        if shape[-1]!=len(value):
+            raise ValueError('Fixed Initializer: input must match shape',len(out),shape)
+        return value
+    return _fixed_initializer
+
+
 class SegmentClassifier(keras.Model):
     """
     """
@@ -823,6 +906,7 @@ class SegmentClassifier(keras.Model):
             output_act_config={},
             seperable_preclassification=False,
             residual_preclassification=False,
+            bias_class_weights=None,
             name=None,
             named_layers=True,
             **stack_config):
@@ -850,12 +934,24 @@ class SegmentClassifier(keras.Model):
         else:
             self.preclassifier=False
         if filters_list:
+            if bias_class_weights:
+                _=bias_class_weights
+                bias_class_weights=tf.convert_to_tensor(
+                    bias_class_weights,
+                    dtype=tf.dtypes.float32)
+                bias_class_weights=bias_class_weights/tf.reduce_sum(bias_class_weights)
+                bias_class_weights=tf.math.log(bias_class_weights)
+                print('FIXED BIAS INITIALIZER:',_,tf.nn.softmax(bias_class_weights))
+                bias_initializer=get_fixed_initializer(bias_class_weights)
+            else:
+                bias_initializer='zeros'
             self.classifier=Conv(
                     filters=filters_list[-1],
                     kernel_size=kernel_size_list[-1],
                     norm=output_norm,
                     norm_config=output_norm_config,
                     act=False,
+                    bias_initializer=bias_initializer,
                     name=self.block_name,
                     named_layers=self.named_layers)
         else:
